@@ -1,7 +1,7 @@
+import logging
 import warnings
 
 import six
-from six import string_types
 from six import StringIO
 from six.moves.urllib.parse import urlparse
 
@@ -9,7 +9,7 @@ from modelcluster.models import ClusterableModel, get_all_child_relations
 
 from django.db import models, connection, transaction
 from django.db.models import Q
-from django.db.models.signals import pre_delete
+from django.db.models.signals import post_save, pre_delete, post_delete
 from django.dispatch.dispatcher import receiver
 from django.http import Http404
 from django.core.cache import cache
@@ -22,10 +22,15 @@ from django.conf import settings
 from django.template.response import TemplateResponse
 from django.utils import timezone
 from django.utils.translation import ugettext_lazy as _
-from django.core.exceptions import ValidationError, ImproperlyConfigured
+from django.core.exceptions import ValidationError, ImproperlyConfigured, ObjectDoesNotExist
 from django.utils.functional import cached_property
 from django.utils.encoding import python_2_unicode_compatible
 
+try:
+    from django.core import checks
+except ImportError:
+    pass
+
 from treebeard.mp_tree import MP_Node
 
 from wagtail.wagtailcore.utils import camelcase_to_underscore, resolve_model_string
@@ -37,6 +42,9 @@ from wagtail.wagtailsearch import index
 from wagtail.wagtailsearch.backends import get_search_backend
 
 
+logger = logging.getLogger('wagtail.core')
+
+
 class SiteManager(models.Manager):
     def get_by_natural_key(self, hostname, port):
         return self.get(hostname=hostname, port=port)
@@ -113,12 +121,6 @@ class Site(models.Model):
                     ]}
                 )
 
-    # clear the wagtail_site_root_paths cache whenever Site records are updated
-    def save(self, *args, **kwargs):
-        result = super(Site, self).save(*args, **kwargs)
-        cache.delete('wagtail_site_root_paths')
-        return result
-
     @staticmethod
     def get_site_root_paths():
         """
@@ -137,6 +139,17 @@ class Site(models.Model):
         return result
 
 
+# Clear the wagtail_site_root_paths from the cache whenever Site records are updated
+@receiver(post_save, sender=Site)
+def clear_site_root_paths_on_save(sender, instance, **kwargs):
+    cache.delete('wagtail_site_root_paths')
+
+
+@receiver(post_delete, sender=Site)
+def clear_site_root_paths_on_delete(sender, instance, **kwargs):
+    cache.delete('wagtail_site_root_paths')
+
+
 PAGE_MODEL_CLASSES = []
 _PAGE_CONTENT_TYPES = []
 
@@ -260,7 +273,7 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
     live = models.BooleanField(default=True, editable=False)
     has_unpublished_changes = models.BooleanField(default=False, editable=False)
     url_path = models.CharField(max_length=255, blank=True, editable=False)
-    owner = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, editable=False, related_name='owned_pages')
+    owner = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, editable=False, on_delete=models.SET_NULL, related_name='owned_pages')
 
     seo_title = models.CharField(verbose_name=_("Page title"), max_length=255, blank=True, help_text=_("Optional. 'Search Engine Friendly' title. This will appear at the top of the browser window."))
     show_in_menus = models.BooleanField(default=False, help_text=_("Whether a link to this page will appear in automatically generated menus"))
@@ -316,21 +329,25 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
     @transaction.atomic  # ensure that changes are only committed when we have updated all descendant URL paths, to preserve consistency
     def save(self, *args, **kwargs):
         update_descendant_url_paths = False
+        is_new = self.id is None
 
-        if self.id is None:
+        if is_new:
             # we are creating a record. If we're doing things properly, this should happen
             # through a treebeard method like add_child, in which case the 'path' field
             # has been set and so we can safely call get_parent
             self.set_url_path(self.get_parent())
         else:
-            # see if the slug has changed from the record in the db, in which case we need to
-            # update url_path of self and all descendants
-            old_record = Page.objects.get(id=self.id)
-            if old_record.slug != self.slug:
-                self.set_url_path(self.get_parent())
-                update_descendant_url_paths = True
-                old_url_path = old_record.url_path
-                new_url_path = self.url_path
+            # Check that we are committing the slug to the database
+            # Basically: If update_fields has been specified, and slug is not included, skip this step
+            if not ('update_fields' in kwargs and 'slug' not in kwargs['update_fields']):
+                # see if the slug has changed from the record in the db, in which case we need to
+                # update url_path of self and all descendants
+                old_record = Page.objects.get(id=self.id)
+                if old_record.slug != self.slug:
+                    self.set_url_path(self.get_parent())
+                    update_descendant_url_paths = True
+                    old_url_path = old_record.url_path
+                    new_url_path = self.url_path
 
         result = super(Page, self).save(*args, **kwargs)
 
@@ -341,8 +358,35 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
         if Site.objects.filter(root_page=self).exists():
             cache.delete('wagtail_site_root_paths')
 
+        # Log
+        if is_new:
+            cls = type(self)
+            logger.info("Page created: \"%s\" id=%d content_type=%s.%s path=%s", self.title, self.id, cls._meta.app_label, cls.__name__, self.url_path)
+
         return result
 
+    @classmethod
+    def check(cls, **kwargs):
+        errors = super(Page, cls).check(**kwargs)
+
+        # Check that foreign keys from pages are not configured to cascade
+        # This is the default Django behaviour which must be explicitly overridden
+        # to prevent pages disappearing unexpectedly and the tree being corrupted
+
+        for field in cls._meta.fields:
+            if isinstance(field, models.ForeignKey) and field.name not in ['page_ptr', 'content_type']:
+                if field.rel.on_delete == models.CASCADE:
+                    errors.append(
+                        checks.Error(
+                            "Field hasn't specified on_delete action",
+                            hint="Set on_delete=models.SET_NULL and make sure the field is nullable.",
+                            obj=field,
+                            id='wagtailcore.E001',
+                        )
+                    )
+
+        return errors
+
     def _update_descendant_url_paths(self, old_url_path, new_url_path):
         cursor = connection.cursor()
         if connection.vendor == 'sqlite':
@@ -412,6 +456,7 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
                 raise Http404
 
     def save_revision(self, user=None, submitted_for_moderation=False, approved_go_live_at=None):
+        # Create revision
         revision = self.revisions.create(
             content_json=self.to_json(),
             user=user,
@@ -422,6 +467,12 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
         self.latest_revision_created_at = revision.created_at
         self.save(update_fields=['latest_revision_created_at'])
 
+        # Log
+        logger.info("Page edited: \"%s\" id=%d revision_id=%d", self.title, self.id, revision.id)
+
+        if submitted_for_moderation:
+            logger.info("Page submitted for moderation: \"%s\" id=%d revision_id=%d", self.title, self.id, revision.id)
+
         return revision
 
     def get_latest_revision(self):
@@ -448,6 +499,8 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
 
             page_unpublished.send(sender=self.specific_class, instance=self.specific)
 
+            logger.info("Page unpublished: \"%s\" id=%d", self.title, self.id)
+
             self.revisions.update(approved_go_live_at=None)
 
     def get_context(self, request, *args, **kwargs):
@@ -509,6 +562,21 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
             if self.url_path.startswith(root_path):
                 return ('' if current_site.id == id else root_url) + reverse('wagtail_serve', args=(self.url_path[len(root_path):],))
 
+    @classmethod
+    def get_indexed_objects(cls):
+        content_type = ContentType.objects.get_for_model(cls)
+        return super(Page, cls).get_indexed_objects().filter(content_type=content_type)
+
+    def get_indexed_instance(self):
+        # This is accessed on save by the wagtailsearch signal handler, and in edge
+        # cases (e.g. loading test fixtures), may be called before the specific instance's
+        # entry has been created. In those cases, we aren't ready to be indexed yet, so
+        # return None.
+        try:
+            return self.specific
+        except ObjectDoesNotExist:
+            return None
+
     @classmethod
     def search(cls, query_string, show_unpublished=False, search_title_only=False, extra_filters={}, prefetch_related=[], path=None):
         # Filters
@@ -650,7 +718,10 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
         new_self.save()
         new_self._update_descendant_url_paths(old_url_path, new_url_path)
 
-    def copy(self, recursive=False, to=None, update_attrs=None, copy_revisions=True):
+        # Log
+        logger.info("Page moved: \"%s\" id=%d path=%s", self.title, self.id, new_url_path)
+
+    def copy(self, recursive=False, to=None, update_attrs=None, copy_revisions=True, keep_live=True):
         # Make a copy
         page_copy = Page.objects.get(id=self.id).specific
         page_copy.pk = None
@@ -659,6 +730,10 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
         page_copy.numchild = 0
         page_copy.path = None
 
+        if not keep_live:
+            page_copy.live = False
+            page_copy.has_unpublished_changes = True
+
         if update_attrs:
             for field, value in update_attrs.items():
                 setattr(page_copy, field, value)
@@ -689,10 +764,13 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
                 revision.page = page_copy
                 revision.save()
 
+        # Log
+        logger.info("Page copied: \"%s\" id=%d from=%d", page_copy.title, page_copy.id, self.id)
+
         # Copy child pages
         if recursive:
             for child_page in self.get_children():
-                child_page.specific.copy(recursive=True, to=page_copy)
+                child_page.specific.copy(recursive=True, to=page_copy, copy_revisions=copy_revisions, keep_live=keep_live)
 
         return page_copy
 
@@ -794,12 +872,10 @@ class Page(six.with_metaclass(PageBase, MP_Node, ClusterableModel, index.Indexed
         return ['/']
 
     def get_sitemap_urls(self):
-        latest_revision = self.get_latest_revision()
-
         return [
             {
                 'location': self.full_url,
-                'lastmod': latest_revision.created_at if latest_revision else None
+                'lastmod': self.latest_revision_created_at
             }
         ]
 
@@ -899,6 +975,11 @@ def unpublish_page_before_delete(sender, instance, **kwargs):
         instance.unpublish(commit=False)
 
 
+@receiver(post_delete, sender=Page)
+def log_page_deletion(sender, instance, **kwargs):
+    logger.info("Page deleted: \"%s\" id=%d", instance.title, instance.id)
+
+
 class Orderable(models.Model):
     sort_order = models.IntegerField(null=True, blank=True, editable=False)
     sort_order_field = 'sort_order'
@@ -917,7 +998,7 @@ class SubmittedRevisionsManager(models.Manager):
 class PageRevision(models.Model):
     page = models.ForeignKey('Page', related_name='revisions')
     submitted_for_moderation = models.BooleanField(default=False)
-    created_at = models.DateTimeField(auto_now_add=True)
+    created_at = models.DateTimeField()
     user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True)
     content_json = models.TextField()
     approved_go_live_at = models.DateTimeField(null=True, blank=True)
@@ -926,6 +1007,12 @@ class PageRevision(models.Model):
     submitted_revisions = SubmittedRevisionsManager()
 
     def save(self, *args, **kwargs):
+        # Set default value for created_at to now
+        # We cannot use auto_now_add as that will override
+        # any value that is set before saving
+        if self.created_at is None:
+            self.created_at = timezone.now()
+
         super(PageRevision, self).save(*args, **kwargs)
         if self.submitted_for_moderation:
             # ensure that all other revisions of this page have the 'submitted for moderation' flag unset
@@ -950,15 +1037,18 @@ class PageRevision(models.Model):
         obj.has_unpublished_changes = self.page.has_unpublished_changes
         obj.owner = self.page.owner
         obj.locked = self.page.locked
+        obj.latest_revision_created_at = self.page.latest_revision_created_at
 
         return obj
 
     def approve_moderation(self):
         if self.submitted_for_moderation:
+            logger.info("Page moderation approved: \"%s\" id=%d revision_id=%d", self.page.title, self.page.id, self.id)
             self.publish()
 
     def reject_moderation(self):
         if self.submitted_for_moderation:
+            logger.info("Page moderation rejected: \"%s\" id=%d revision_id=%d", self.page.title, self.page.id, self.id)
             self.submitted_for_moderation = False
             self.save(update_fields=['submitted_for_moderation'])
 
@@ -995,8 +1085,12 @@ class PageRevision(models.Model):
         if page.live:
             page_published.send(sender=page.specific_class, instance=page.specific)
 
+            logger.info("Page published: \"%s\" id=%d revision_id=%d", page.title, page.id, self.id)
+        elif page.go_live_at:
+            logger.info("Page scheduled for publish: \"%s\" id=%d revision_id=%d go_live_at=%s", page.title, page.id, self.id, page.go_live_at.isoformat())
+
     def __str__(self):
-        return '"' + unicode(self.page) + '" at ' + unicode(self.created_at)
+        return '"' + six.text_type(self.page) + '" at ' + six.text_type(self.created_at)
 
 
 PAGE_PERMISSION_TYPE_CHOICES = [
