@@ -10,5 +10,5 @@ urlpatterns = [
     # a '/'. If a trailing slash is not present, we leave CommonMiddleware to
     # handle it as usual (i.e. redirect it to the trailing slash version if
     # settings.APPEND_SLASH is True)
-    url(r'^((?:[\w\-]+/)*)$', views.serve)
+    url(r'^((?:[\w\-]+/)*)$', views.serve, name='wagtail_serve')
 ]
