@@ -4,11 +4,23 @@ from django.utils.translation import ugettext as _
 
 from wagtail.wagtailimages.models import get_image_model
 from wagtail.wagtailimages.formats import get_image_formats
+from wagtail.wagtailimages.fields import WagtailImageField
+
+
+# Callback to allow us to override the default form field for the image file field
+def formfield_for_dbfield(db_field, **kwargs):
+    # Check if this is the file field
+    if db_field.name == 'file':
+        return WagtailImageField(**kwargs)
+
+    # For all other fields, just call its formfield() method.
+    return db_field.formfield(**kwargs)
 
 
 def get_image_form():
     return modelform_factory(
         get_image_model(),
+        formfield_callback=formfield_for_dbfield,
         # set the 'file' widget to a FileInput rather than the default ClearableFileInput
         # so that when editing, we don't get the 'currently: ...' banner which is
         # a bit pointless here
@@ -21,16 +33,6 @@ def get_image_form():
         })
 
 
-def get_image_form_for_multi():
-    # exclude the file widget
-    return modelform_factory(get_image_model(), exclude=('file',), widgets={
-        'focal_point_x': forms.HiddenInput(attrs={'class': 'focal_point_x'}),
-        'focal_point_y': forms.HiddenInput(attrs={'class': 'focal_point_y'}),
-        'focal_point_width': forms.HiddenInput(attrs={'class': 'focal_point_width'}),
-        'focal_point_height': forms.HiddenInput(attrs={'class': 'focal_point_height'}),
-    })
-
-
 class ImageInsertionForm(forms.Form):
     """
     Form for selecting parameters of the image (e.g. format) prior to insertion
